#!/usr/bin/python
# -*- coding: UTF-8 -*-

from dli.table import TableSchema
from dli.column import Column
from dli.exception import DliException
from dli.dli_client import DliClient

PROXY = 'https://ip:port'


def init_aksk_dli_client():
    auth_mode = "aksk"
    region = 'cn-north-1'
    project_id = 'xxxx'
    ak = 'xxxx'
    sk = 'xxxx'

    dli_client = DliClient(auth_mode=auth_mode, region=region, project_id=project_id,
                           ak=ak, sk=sk, proxy=PROXY)
    return dli_client


def init_token_dli_client():
    auth_mode = "token"
    region = 'cn-north-1'
    project_id = 'xxxx'
    account = 'huaweicloud_account'
    user = 'iam_user_name'
    password = 'xxxx'

    dli_client = DliClient(auth_mode=auth_mode, region=region, project_id=project_id,
                           account=account, user=user, password=password, proxy=PROXY)
    return dli_client


def list_all_queues(dli_client):
    try:
        queues = dli_client.list_queues()
    except DliException as e:
        print(e)
        return

    for queue in queues:
        print(queue.name)


def create_db(dli_client):
    try:
        db = dli_client.create_database('db_for_test')
    except DliException as e:
        print(e)
        return

    print(db)


def list_all_dbs(dli_client):
    try:
        dbs = dli_client.list_databases()
    except DliException as e:
        print(e)
        return

    for db in dbs:
        print(db)


def create_dli_tbl(dli_client, db_name, tbl_name):
    cols = [
        Column('col_1', 'string'),
        Column('col_2', 'string'),
        Column('col_3', 'smallint'),
        Column('col_4', 'int'),
        Column('col_5', 'bigint'),
        Column('col_6', 'double'),
        Column('col_7', 'decimal(10,0)'),
        Column('col_8', 'boolean'),
        Column('col_9', 'date'),
        Column('col_10', 'timestamp')
    ]
    sort_cols = ['col_1']
    tbl_schema = TableSchema(tbl_name, cols, sort_cols)
    try:
        table = dli_client.create_dli_table(db_name, tbl_schema)
    except DliException as e:
        print(e)
        return

    print(table)


def create_obs_tbl(dli_client, db_name, tbl_name):
    cols = [
        Column('col_1', 'string'),
        Column('col_2', 'string'),
        Column('col_3', 'smallint'),
        Column('col_4', 'int'),
        Column('col_5', 'bigint'),
        Column('col_6', 'double'),
        Column('col_7', 'decimal(10,0)'),
        Column('col_8', 'boolean'),
        Column('col_9', 'date'),
        Column('col_10', 'timestamp')
    ]
    tbl_schema = TableSchema(tbl_name, cols)
    try:
        table = dli_client.create_obs_table(db_name, tbl_schema,
                                            's3a://bucket/obj',
                                            'csv')
    except DliException as e:
        print(e)
        return

    print(table)


def list_all_tbls(dli_client, db_name):
    try:
        tbls = dli_client.list_tables(db_name, with_detail=True)
    except DliException as e:
        print(e)
        return

    for tbl in tbls:
        print(tbl.name)


def import_data(dli_client, db_name, tbl_name, queue_name):
    try:
        job_id, status = \
            dli_client.import_table(tbl_name, db_name,
                                    's3a://bucket/obj/data.csv',
                                    'csv', queue_name=queue_name)
    except DliException as e:
        print(e)
        return

    print(job_id)
    print(status)


def export_data(dli_client, db_name, tbl_name, queue_name):
    try:
        job_id, status = dli_client.export_table(tbl_name, db_name,
                                                 's3a://bucket/obj',
                                                 queue_name=queue_name)
    except DliException as e:
        print(e)
        return

    print(job_id)
    print(status)


def run_sql(dli_client, db_name, queue_name):
    # execute SQL
    try:
        sql_job = dli_client.execute_sql('select * from tbl_dli_for_test', db_name)
        result_set = sql_job.get_result()
    except DliException as e:
        print(e)
        return

    if result_set.row_count == 0:
        return

    for row in result_set:
        print(row)

    # export the query result to obs
    try:
        status = sql_job.export_result('s3a://bucket/obj',
                                       queue_name=queue_name)
    except DliException as e:
        print(e)
        return

    print(status)


def delete_tbls(dli_client, db_name):
    try:
        tbls = dli_client.list_tables(db_name)
        for tbl in tbls:
            dli_client.delete_table(db_name, tbl.name)
    except DliException as e:
        print(e)
        return


def delete_db(dli_client, db_name):
    try:
        dli_client.delete_database(db_name)
    except DliException as e:
        print(e)
        return


def main():
    db_name = 'db_for_test'
    queue_name = 'queue_for_test'
    dli_tbl_name = 'tbl_dli_for_test'
    obs_tbl_name = 'tbl_obs_for_test'

    dli_client = init_aksk_dli_client()

    list_all_queues(dli_client)

    create_db(dli_client)

    list_all_dbs(dli_client)

    create_dli_tbl(dli_client, db_name, dli_tbl_name)

    create_obs_tbl(dli_client, db_name, obs_tbl_name)

    list_all_tbls(dli_client, db_name)

    import_data(dli_client, db_name, dli_tbl_name, queue_name)

    export_data(dli_client, db_name, dli_tbl_name, queue_name)

    run_sql(dli_client, db_name, queue_name)

    delete_tbls(dli_client, db_name)

    delete_db(dli_client, db_name)


if __name__ == '__main__':
    main()
